import os
import MySQLdb
import sys

class CFDB_UserDatabase():
	host = "127.0.0.1"
# Add user database credentials here.
	user = ""
	passwd = ""
	db = ""
	__databaseInitialized = 0
	
	def __init__(self):
		self.conn = MySQLdb.connect (self.host, self.user, self.passwd, self.db)
		self.cursor = self.conn.cursor()

	def ClearTables(self):
		cypher = "DROP TABLE IF EXISTS Picks,UserPicks,Users,WeeklyResults"

		try:
			self.cursor.execute(cypher)
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def InitializeDatabase(self):
		if self.ClearTables():
			return -1

		print("Initializing database...", flush=True, end='')
		try:
			if self.CreatePicksTable():
				return -1
			if self.CreateUserPicksTable():
				return -1
			if self.CreateUsersTable():
				return -1
			if self.CreateWeeklyResultsTable():
				return -1
			print("done", flush=True)
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def CreatePicksTable(self):
		createPicksTable = 'CREATE TABLE Picks ({0},{1},{2},{3},{4},{5},{6})'.format(
			"GameID INT NOT NULL PRIMARY KEY",
			"Winner INT NOT NULL",
			"Week INT NOT NULL",
			"Season CHAR(64) NOT NULL",
			"Type CHAR(128) NOT NULL",
			"Points INT NOT NULL",
			"Underdog INT NOT NULL",
			"GameIndex INT NOT NULL"
		)
		
		try:
			self.cursor.execute(createPicksTable)
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return

	def CreateUserPicksTable(self):
		createUserPicksTable = 'CREATE TABLE UserPicks({0},{1},{2},{3},{4},{5})'.format(
			"PickID INT NOT NULL PRIMARY KEY AUTO_INCREMENT",
			"GameID INT NOT NULL",
			"UserID INT NOT NULL",
			"Winner INT NOT NULL",
			"Week INT NOT NULL",
			"Season CHAR(64) NOT NULL",
			"GameIndex INT NOT NULL"
		)

		try:
			self.cursor.execute(createUserPicksTable)
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return

	def CreateUsersTable(self):
		createUsersTable = 'CREATE TABLE Users({0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11},{12},{13},{14},{15},{16},{17},{18},{19},{20})'.format(
			"UserID INT NOT NULL PRIMARY KEY AUTO_INCREMENT",
			"FirstName VARCHAR(64)",
			"LastName VARCHAR(64)",
			"UserName VARCHAR(64)",
			"Password CHAR(64)",
			"Salt CHAR(16)",
			"Email VARCHAR(64)",
			"WeeklyEmails TINYINT",
			"MassEmails TINYINT",
			"RegularPoints INT",
			"ConferencePoints INT",
			"SeasonWins INT",
			"ConferenceID INT",
			"OverallWins INT",
			"OverallLosses INT",
			"OverallTies INT",
			"ConferenceWins INT",
			"ConferenceLosses INT",
			"ConferenceTies INT",
			"UserType TINYINT",
			"Theme INT"
		)

		try:
			self.cursor.execute(createUsersTable)
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return

	def CreateWeeklyResultsTable(self):
		createWeeklyResultsTable = 'CREATE TABLE WeeklyResults({0},{1},{2},{3})'.format(
			"UserID INT",
			"Points INT",
			"Week INT",
			"Season CHAR(64)"
		)

		try:
			self.cursor.execute(createWeeklyResultsTable)
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return

	def GetAllUserIds(self):
		sqlCommand = "SELECT UserID FROM Users";
		results = []
		try:
			self.cursor.execute( sqlCommand )
			users = self.cursor.fetchall()
			for user in users:
				results.append( user[0] )
			return results
		except MySQLdb.Error as e:
			print('GetAllUserIds SQL Error[%d]: %s' % ( e.args[0], e.args[1] ))
			return None

	def GetUserRegularPoints(self, userId):
		sqlCommand = "SELECT regularPoints FROM Users WHERE id={0}".format( userId )
	
		try:
			self.cursor.execute( sqlCommand )
			points = self.cursor.fetchone()
			return points[0]
		except MySQLdb.Error as e:
			print('GetUserRegularPoints SQL Error [%d}: %s' % (e.args[0], e.args[1] ))
			return None

	def SetUserSeasonPoints( self, userId, seasonPoints ):
		sqlCommand = "UPDATE Users SET SeasonWins={0} WHERE UserID={1}".format( seasonPoints, userId )
		try:
			self.cursor.execute( sqlCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1]'.format(__name__, e))
			return -1

	def SetUserRegularPoints( self, userId, points ):
		sqlCommand = "UPDATE Users SET RegularPoints={0} WHERE UserID={1}".format( points, userId )
		try:
			self.cursor.execute( sqlCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1]'.format(__name__, e))
			return -1
	
	def GetUserConferencePoints(self, userId):
		sqlCommand = "SELECT ConferencePoints FROM Users WHERE UserID={0}".format( userId )
	
		try:
			self.cursor.execute( sqlCommand )
			points = self.cursor.fetchone()
			return points[0]
		except MySQLdb.Error as e:
			print('Error in {0}: {1]'.format(__name__, e))
			return None

	def SetUserConferencePoints( self, userId, points ):
		sqlCommand = "UPDATE Users SET ConferencePoints={0} WHERE UserID={1}".format( points, userId )

		try:
			self.cursor.execute( sqlCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1]'.format(__name__, e))
			return -1

	def GetUserFirstNameById( self, userId ):
		sqlCommand = "SELECT firstName FROM Users WHERE id={0}".format( userId )
	
		try:
			self.cursor.execute( sqlCommand )
			firstName = self.cursor.fetchone()
			return firstName[0]
		except MySQLdb.Error as e:
			print('GetUserRegularPoints SQL Error [%d}: %s' % (e.args[0], e.args[1] ))
			return None

	def SetUserOverallChallengeRecord( self, userId, userWins, userLosses, userTies ):
		sqlCommand = "UPDATE Users SET OverallWins={0},OverallLosses={1},OverallTies={2} WHERE UserID={3}".format( userWins, userLosses, userTies, userId )

		try:
			self.cursor.execute( sqlCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('SetUserOverallChallengeRecord SQL Error[%d]: %s' % ( e.args[0], e.args[1] ))
			return None

	def SetUserConferenceChallengeRecord( self, userId, userWins, userLosses, userTies ):
		sqlCommand = "UPDATE Users SET ConferenceWins={0},ConferenceLosses={1},ConferenceTies={2} WHERE UserID={3}".format( userWins, userLosses, userTies, userId )

		try:
			self.cursor.execute( sqlCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('SetUserConferenceChallengeRecord SQL Error[%d]: %s' % ( e.args[0], e.args[1] ))
			return None

	def GetUserConferenceById( self, userId ):
		sqlCommand = "SELECT conferenceId FROM Users WHERE id={0}".format( userId )

		try:
			self.cursor.execute( sqlCommand )
			conferenceId = self.cursor.fetchone()
			return conferenceId[0]
		except MySQLdb.Error as e:
			print('GetUserConferenceById SQL Error[%d]: %s' % ( e.args[0], e.args[1] ))
			return None

	def SetPickWinnerByGameId( self, gameId, winner ):
		setWinnerCommand = 'UPDATE Picks SET Winner={0} WHERE GameID={1}'. format( winner, gameId )
		try:
			self.cursor.execute( setWinnerCommand )
			self.conn.commit()
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return None

	def GetPicksByWeek(self, week, season):
		gameIDList = []
		getPicksByWeekCommand = 'SELECT GameID FROM Picks WHERE Week={0} AND Season="{1}"'.format(week, season)
		try:
			self.cursor.execute(getPicksByWeekCommand)
			gameIds = self.cursor.fetchall()
			for game in gameIds:
				gameIDList.append( game[0] )
			return gameIDList
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return None

	def GetPicksAndPointsByWeek( self, week, season ):
		picks = {}
		sqlCommand = "SELECT gameId,points FROM Picks WHERE Week={0} AND Season='{1}'".format( week, season )

		try:
			self.cursor.execute( sqlCommand )
			results = self.cursor.fetchall()
			for result in results:
				picks[int(result[0])] = int( result[1] )
			return picks
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return None

	def GetPickType( self, gameId ):
		sqlCommand = 'SELECT type FROM Picks WHERE gameId={0}'.format( gameId )
	
		try:
			self.cursor.execute( sqlCommand )
			gameType = self.cursor.fetchone()
			return gameType
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return None

	def GetUserCorrectPicksByWeek( self, userId, week, season ):
		sqlCommand = 'SELECT GameID FROM (SELECT Picks.GameID, Picks.Winner FROM Picks WHERE Week={0} ' \
					'UNION ALL SELECT UserPicks.GameID, UserPicks.Winner FROM UserPicks WHERE Week={1} ' \
					'AND Season="{2}" AND UserID={3}) t GROUP BY GameID, Winner HAVING COUNT(*) = 2 ORDER BY GameID'.format(
						week,
						week,
						season,
						userId
						)
		try:
			self.cursor.execute( sqlCommand )
			results = self.cursor.fetchall()
			if results is None:
				return None
			return results
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def GetUserPicksByWeek( self, userId, week, season ):
		sqlCommand = "SELECT GameId,winner FROM UserPicks WHERE userId={0} AND week={1} AND Season={2}".format( userId, week, season )
		picks = []

		try:
			self.cursor.execute( sqlCommand )
			results = self.cursor.fetchall()
			for result in results:
				picks.append( (result[0], result[1] ) )
			return picks
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def GetWinnerByGame( self, gameId ):
		sqlCommand = "SELECT winner FROM Picks WHERE gameId={0}".format( gameId )

		try:
			self.cursor.execute( sqlCommand )
			results = self.cursor.fetchone()
			if results == False or results == None:
				return 0
			return results[0]
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def GetGameType( self, gameId ):
		sqlCommand = "SELECT type FROM Picks WHERE gameId={0}".format( gameId )

		try:
			self.cursor.execute( sqlCommand )
			results = self.cursor.fetchone()
			return results[0]
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def SetUserWeeklyPoints( self, userId, week, season, points ):
		sqlInsertCommand = "INSERT INTO WeeklyResults ( userId, points, week, Season ) VALUES ( {0}, {1}, {2}, '{3}' )".format( userId, points, week, season )
		sqlUpdateCommand = "UPDATE WeeklyResults SET points={0} WHERE userId={1} AND week={2} AND Season='{3}'".format( points, userId, week, season )
		sqlSelectCommand = "SELECT points FROM WeeklyResults WHERE userId={0} AND week={1} AND Season='{2}'".format( userId, week, season )

		try:
			self.cursor.execute( sqlSelectCommand )
			result = self.cursor.fetchone()
			if result == None:
				self.cursor.execute( sqlInsertCommand )
			else:
				self.cursor.execute( sqlUpdateCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
		#	print(sqlSelectCommand)
		#	print(sqlInsertCommand)
		#	print(sqlUpdateCommand)
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def GetUserWeeklyPoints( self, userId, week, season ):
		sqlCommand = "SELECT points FROM WeeklyResults WHERE userId={0} AND week={1} AND Season={2}".format( userId, week, season )

		try:
			self.cursor.execute( sqlCommand )
			result = self.cursor.fetchone()
			if result == None or result[0] == None:
				return 0
			else:
				return result[0]
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def GetLowestScoreByWeek( self, week, season, userId ):
		sqlCommand = "SELECT MIN(NULLIF(points,0)) FROM WeeklyResults WHERE week={0} AND Season={1}".format( week, season )

		try:
			self.cursor.execute( sqlCommand )
			result = self.cursor.fetchone()
			if result == None or result[0] == None:
				return 0
			else:
				return result[0]
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def GetAllChallenges( self ):
		sqlCommand = "SELECT * from Challenges"

		try:
			self.cursor.execute( sqlCommand )
			challenges = self.cursor.fetchall()
			return challenges
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return None

	def GetAllUserChallenges( self, userId ):
		sqlCommand = "SELECT * FROM Challenges WHERE challenger1={0} OR challenger2={1}".format( userId, userId )

		try:
			self.cursor.execute( sqlCommand )
			challenges = self.cursor.fetchall()
			return challenges
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return None


	def SetChallenger1Points( self, challenge, points ):
		sqlCommand = "UPDATE Challenges Set c1_points={0} WHERE id={1}".format( points, challenge )

		try:
			self.cursor.execute( sqlCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def SetChallenger2Points( self, challenge, points ):
		sqlCommand = "UPDATE Challenges Set c2_points={0} WHERE id={1}".format( points, challenge )

		try:
			self.cursor.execute( sqlCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def SetChallengeWinner( self, challenge, winner ):
		sqlCommand = "UPDATE Challenges SET winner={0} WHERE id={1}".format( winner, challenge )

		try:
			self.cursor.execute( sqlCommand )
			self.conn.commit()
			return 0
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def GetPicksWeek( self ):
		sqlCommand = "SELECT MAX( week ) FROM Picks"

		try:
			self.cursor.execute( sqlCommand )
			currentWeek = self.cursor.fetchone()
			if currentWeek == None or currentWeek[0] == None:
				return 1
			else:
				return currentWeek[0]
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def GetChallengeWeek( self ):
		currentWeek = self.GetPicksWeek()
		sqlCommand = "SELECT * FROM Picks WHERE week={0} and winner=0".format( currentWeek )

		try:
			self.cursor.execute( sqlCommand )
			results = self.cursor.fetchall()
			if results == None or len( results ) == 0:
				return currentWeek + 1
			else:
				return currentWeek
		except MySQLdb.Error as e:
			print('Error in {0}: {1}'.format(sys._getframe().f_code.co_name, e))
			return -1

	def UpdatePicks( self, games, week, season ):
		picks = self.GetPicksByWeek( week, season )
		for pickID in picks:
			for game in games:
				if game.id == pickID:
					if game.completed == True:
						if game.home_points > game.away_points:
							self.SetPickWinnerByGameId(game.id, game.home_id)
						elif game.away_points > game.home_points:
							self.SetPickWinnerByGameId(game.id, game.away_id)
